@@ -291,7 +291,7 @@ get_session_token <- function(profile_name = NULL,
 assume_role <- function(profile_name = NULL,
                         region_name = NULL,
                         role_arn = NULL,
-                        role_session_name= sprintf("paws-athena-session-%s", as.integer(Sys.time())),
+                        role_session_name= sprintf("noctua-session-%s", as.integer(Sys.time())),
                         duration_seconds = 3600L,
                         set_env = FALSE){
   stopifnot(is.null(profile_name) || is.character(profile_name),
